//
// ComplexMenuItem.cs
//
// Author:
//   Aaron Bockover <abockover@novell.com>
//   Gabriel Burt <gburt@novell.com>
//
// Copyright (C) 2007 Novell, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using Gtk;

namespace FSpot.Widgets
{
	public class ComplexMenuItem : MenuItem
	{
		private bool is_selected = false;

		public event EventHandler Changed;
		public void OnChanged (object o, EventArgs e)
		{
			EventHandler handler = Changed;
			if (handler != null)
				handler (o, e);
		}

		public ComplexMenuItem() : base()
		{
		}

		// Override OnAdded and OnRemoved so we can work with Gtk.Action/Gtk.UIManager
		// which otherwise would try to replace our child with a Label.
		private bool first_add = true;
		protected override void OnAdded (Widget widget)
		{
			if (first_add) {
				first_add = false;
				base.OnAdded (widget);
			}
		}

		protected override void OnRemoved (Widget widget)
		{
		}

		protected override void OnSelected()
		{
			base.OnSelected();
			is_selected = true;
		}

		protected override void OnDeselected()
		{
			base.OnDeselected();
			is_selected = false;
		}

		protected override void OnParentSet(Widget previous_parent)
		{
			if(previous_parent != null) {
				previous_parent.KeyPressEvent -= OnKeyPressEventProxy;
			}

			if(Parent != null) {
				Parent.KeyPressEvent += OnKeyPressEventProxy;
			}
		}

		[GLib.ConnectBefore]
		private void OnKeyPressEventProxy(object o, KeyPressEventArgs args)
		{
			if(!IsSelected) {
				return;
			}

			switch(args.Event.Key) {
			case Gdk.Key.Up:
			case Gdk.Key.Down:
			case Gdk.Key.Escape:
				return;
			}

			args.RetVal = OnKeyPressEvent(args.Event);
		}

		protected override bool OnKeyPressEvent(Gdk.EventKey evnt)
		{
			return false;
		}

		protected bool IsSelected {
			get { return is_selected; }
		}
	}
}
